#! /usr/bin/python

# --- bugs (in subclasses too)
# 
# - toggleSign bug: "123 + 34" + "+/-" = "-123 + 34"
#

from __future__ import division # So that 8/3 will be 2.6666 and not 2

class AbstractCalculator(object):
    def __init__(self, debug=False):
        
        self.debug = debug
        self._display = '0'
        self.result = False # true if display shows a result
        self.keyPress = ''
        self.history = ['0']
        self.keys = {'1':   self.addToDisplay,
                     '2':   self.addToDisplay,
                     '3':   self.addToDisplay,
                     '4':   self.addToDisplay,
                     '5':   self.addToDisplay,
                     '6':   self.addToDisplay,
                     '7':   self.addToDisplay,
                     '8':   self.addToDisplay,
                     '9':   self.addToDisplay,
                     '0':   self.addToDisplay,
                     '.':   self.addToDisplay,
                     '+':   self.addToDisplay,
                     '-':   self.addToDisplay,
                     '*':   self.addToDisplay,
                     '/':   self.addToDisplay,
                     '+/-': self.toggleSign,
                     '=':   self.evalDisplay,
                     'CE':  self.cancelEntry,
                     'C':   self.clearDisplay,
                     'OFF': self.quitApp}

        self.keyLayout = [[ '1',  '2',  '3',  '+'],
                          [ '4',  '5',  '6',  '-'],
                          [ '7',  '8',  '9',  '*'],
                          [ '0',  '.', '+/-', '/'],
                          ['OFF', 'C',  'CE', '=']]

    def getDisplay(self):
        return self._display

    def setDisplay(self, value):
        self._display = value

    display = property(getDisplay, setDisplay)

    def addToDisplay(self):
        item = self.keyPress
        if self.result is True and item.isdigit():
            newDisplay = item
            self.result = False
        elif item.isdigit() or item == '.':
            if self.display == '0': self.display = ''
            newDisplay = '%s%s' % (self.display, item)
        else:
            if self.display[-1] != ' ': # don't allow more than one operator
                newDisplay = '%s %s ' % (self.display, item)
        self.updateDisplay(newDisplay)

    def evalDisplay(self): # key '='
        try:
            n = str(eval(self.display))
        except:
            n = 'ERROR'
        self.updateDisplay(n)

    def cancelEntry(self): # key 'CE'
        if len(self.history) == 1:
            self.history.insert(0, '0')
        self.history.pop()
        self.updateDisplay(self.history[-1], True)
    
    def clearDisplay(self): # key 'C'
        self.updateDisplay('0')
    
    def keyAction(self, keyString=None):
        if keyString:
            self.keyPress = keyString
            self.keys[keyString]()
        if self.debug is True:
            print 'MyCalc::keyAction():\t%s\t[%s]' % (keyString, self.display)

    def toggleSign(self): # key '+/-'
        if self.display[0] == '-':
            newDisplay = self.display[1:]
        else:
            newDisplay = '-%s' % self.display
        self.updateDisplay(newDisplay)

    def updateDisplay(self, newDisplay, skipHistory=False):
        self.display = newDisplay
        if not skipHistory:
            self.history.append(newDisplay)

    def quitApp(self): # key 'OFF'
        pass

    def buttonPress(self, event):
        if event in self.keys:
            self.keyAction(event)

def main():    
    input = ['2', 'd', '5', '1', '+', '3', '3', 'CE', 'CE', '5', '=', ] # 256
    #input = ['8', '/', 's', '3', '='] # 2.66666666667
    calc = AbstractCalculator(True)
    print '\t', calc.display
    for i in input:
        calc.buttonPress(i)
    print '\t', calc.display

if __name__ == '__main__':
    main()
