#! /usr/bin/python

### References
###
### 1.
### 
### http://wiki.wxpython.org/index.cgi/CalculatorDemo
### and/or
### http://pythonwise.blogspot.com/2006/05/wxpython-calculator-in-50-lines-of.html
###
### 2.
### 
### http://www.devshed.com/c/a/Python/Designing-a-Calculator-in-wxPython/

import wx
from myCalc import AbstractCalculator

class WxCalc(wx.Frame, AbstractCalculator):
    keyCode2label = {wx.WXK_RETURN: '=',  # return
                     wx.WXK_BACK: 'C',    # backspace
                     wx.WXK_DELETE: 'C',  # delete
                     17: 'OFF'  # q
                     }
    
    def __init__(self, debug=False):
        # init
        AbstractCalculator.__init__(self, debug)
        self.app = wx.PySimpleApp()

        # main frame
        wx.Frame.__init__(self, None, -1, "Simple WxP Calc")
        sizer = wx.BoxSizer(wx.VERTICAL) # Main vertical sizer

        # display
        self._display = wx.TextCtrl(self, -1, '',
                                    style = wx.TE_READONLY | wx.TE_RIGHT)
        sizer.Add(self._display, 0, wx.EXPAND) # Add to main sizer

        # keys
        gsizer = wx.GridSizer(4, 4)
        for row in self.keyLayout:
            for key in row:
                b = wx.Button(self, -1, key)
                b.Bind(wx.EVT_BUTTON,
                       lambda e, k=key: self.keyAction(k))
                b.SetFocus()  # linux # needed on windows?
                gsizer.Add(b)
        sizer.Add(gsizer, 1, wx.EXPAND)

        self.SetSizer(sizer)
        sizer.Fit(self)

        # key bindings
        self.Bind(wx.EVT_CHAR, self.buttonPress)
        self.display = '0'

        # go
        self.Show()
        self.app.MainLoop()

    def getDisplay(self):
        return self._display.GetValue()

    def setDisplay(self, value):
        self._display.SetValue(value)

    display = property(getDisplay, setDisplay)

    def quitApp(self):
        self.Destroy()

    def buttonPress(self, event):
        c = event.GetKeyCode()
        k = chr(c)
        if self.debug:
            print 'MyWxCalc::buttonPress:\tc: %s; k: %s' % (c, k)
        if k not in '1234567890+-*/.=':
            k = self.keyCode2label.get(c)
        self.keyAction(k)


if __name__ == "__main__":
    WxCalc(True)
